import abc
import json
import os
from configparser import ConfigParser, NoOptionError
from pathlib import Path
from typing import Any, Dict, List, Optional, Callable

import requests

from kttool.logger import color_green, log, log_cyan, log_red
from kttool.utils import (
    HEADERS, KATTIS_RC_URL, MAP_TEMPLATE_TO_PLANG, PLanguage, ask_with_default
)


class ConfigError(Exception):
    pass


def require_login(fn: Callable) -> Callable:
    def inner(self: 'Action', *args, **kwargs):
        did_login = False
        if not self.is_logged_in:
            did_login = True
            self.login()
        ret = fn(self, *args, **kwargs)
        if did_login:
            # poor man way to "timeout" the login attempt and invalidate the cookies
            self.is_logged_in = False
        return ret

    return inner


class Action(abc.ABC):
    ''' 
    Base class for handle general command.
    Handle loading up .kattisrc config file
    '''
    REQUIRED_CONFIG = False

    cwd: Path
    config_path: Path
    cfg: Optional[ConfigParser]
    cookies: Any
    kt_config: Path

    file_name: Optional[Path]
    lang: Optional[str]
    pre_script: Optional[str]
    script: Optional[str]
    post_script: Optional[str]
    is_logged_in: bool

    __slots__ = 'cwd', 'config_path', 'cfg', 'cookies', 'kt_config', 'file_name', 'lang', \
        'pre_script', 'script', 'post_script', 'is_logged_in'

    def __init__(self, *, cwd: Optional[Path] = None):
        self.config_path = Path.home() / '.kattisrc'  # kattis config file
        self.kt_config = Path.home() / '.ktconfig'  # kt tool file
        self.cfg = None
        self.cwd = cwd or Path().absolute()
        self.cookies = None

        self.file_name = None
        self.lang = None
        self.pre_script = None
        self.script = None
        self.post_script = None
        self.is_logged_in = False

    def get_url(self, option: str, default: str = '') -> str:
        """ Get appropriate urls from kattisrc file

        Parameters
        ----------
        option : str
            parameter to get from katticrc config file
        default : str, optional
            fallback value if option is not present, by default ''

        Returns
        -------
        str
            Full url path to the required attr
        """
        if self.cfg.has_option('kattis', option):
            return self.cfg.get('kattis', option)

        kattis_host = self.cfg.get('kattis', 'hostname')
        return f'https://{kattis_host}/{default}'

    def read_config_from_file(self) -> None:
        """ kttool deals with 2 config files:
        - kattisrc: provided by official kattis website, provide domain name and general urls
        - ktconfig: handle templates by kttool

        Raises
        ------
        RuntimeError
            If config file is invalid
        ConfigError
            If config file is invalid
        """

        # Initialize ktconfig file if file doesnt exist
        if not self.kt_config.is_file():
            with open(self.kt_config, 'w') as f:
                f.write('{}\n')

        self.cfg = ConfigParser()
        if not self.config_path.is_file():
            raise RuntimeError(
                f'No valid config file at {self.config_path}. '
                f'Please download it at {KATTIS_RC_URL}'
            )

        self.cfg.read(self.config_path)
        username = self.cfg.get('user', 'username')
        password = token = None
        try:
            password = self.cfg.get('user', 'password')
        except NoOptionError:
            pass
        try:
            token = self.cfg.get('user', 'token')
        except NoOptionError:
            pass
        if password is None and token is None:
            raise ConfigError(
                '''\
        Your .kattisrc file appears corrupted. It must provide a token (or a
        KATTIS password).
        Please download a new .kattisrc file'''
            )
        log(f'Username: {color_green(username)}')

    def login(self) -> None:
        """ Try to login and obtain cookies from succesful signin

        Raises
        ------
        RuntimeError
            If login fails
        """

        username = self.cfg.get('user', 'username')
        password = token = ''
        try:
            password = self.cfg.get('user', 'password')
        except NoOptionError:
            pass
        try:
            token = self.cfg.get('user', 'token')
        except NoOptionError:
            pass
        login_url = self.get_url('loginurl', 'login')
        login_args = {'user': username, 'script': 'true'}
        if password:
            login_args['password'] = password
        if token:
            login_args['token'] = token
        login_reply = self._request_post(login_url, data=login_args)

        if login_reply.status_code != 200:
            if login_reply.status_code == 403:
                err = 'Incorrect username or password/token (403)'
            elif login_reply.status_code == 404:
                err = 'Incorrect login URL (404)'
            else:
                err = f'Status code: {login_reply.status_code}'
            raise RuntimeError(f'Login failed. {err}')
        self.cookies = login_reply.cookies
        self.is_logged_in = True

    def _get_problem_id(self) -> str:
        # Assuming user is in the folder with the name of the problem id
        return self.cwd.name

    def _request_get(self, uri: str) -> requests.Response:
        return requests.get(uri, cookies=self.cookies, headers=HEADERS)

    def _request_post(self, uri: str, *args, **kwargs) -> requests.Response:
        return requests.post(
            uri, *args, **kwargs, cookies=self.cookies, headers=HEADERS
        )

    def get_problem_url(self) -> str:
        domain = f"https://{self.get_url('hostname')}"
        problem_id = self._get_problem_id()
        return os.path.join(domain, 'problems', problem_id)

    def _detect_code_files(self) -> bool:
        """ Confirm the executable file if there is multiple files that are runnable in current folder

        Raises
        ------
        RuntimeError
            If no executable code detected
        """
        acceptable_file_ext: Dict[str, PLanguage] = {}
        opt = 0
        i = 0
        existed_templates = self.load_kt_config()

        for k in existed_templates.keys():
            acceptable_file_ext[MAP_TEMPLATE_TO_PLANG[k].extension
                               ] = MAP_TEMPLATE_TO_PLANG[k]

        files = [x for x in self.cwd.iterdir() if x.is_file()]
        runnable_files: List[Path] = []
        for f in files:
            if f.suffix[1:] in acceptable_file_ext:
                runnable_files.append(f)

        if not runnable_files:
            log_red('No executable code files detected')
            return False

        if len(runnable_files) > 1:
            log_cyan('Choose a file to run')
            for i in range(len(runnable_files)):
                log(f'  {i}: {runnable_files[i].file_name}')
            opt = int(ask_with_default('Choose a file to run', default_val='0'))
            if not (0 <= opt < len(runnable_files)):
                log_red('Invalid option chosen')
                return False

        self.file_name = runnable_files[opt]
        alias = acceptable_file_ext[runnable_files[opt].suffix[1:]].alias
        self.lang = acceptable_file_ext[runnable_files[opt].suffix[1:]
                                       ].full_name

        file_name = self.file_name.stem
        self.pre_script = existed_templates.get(alias,
                                                {}).get('pre_script').replace(
                                                    '$%file%$', file_name
                                                )
        self.script = existed_templates.get(alias, {}).get('script').replace(
            '$%file%$', file_name
        )
        self.post_script = existed_templates.get(alias,
                                                 {}).get('post_script').replace(
                                                     '$%file%$', file_name
                                                 )
        return True

    def load_kt_config(self) -> dict:
        if not self.kt_config.is_file():
            with open(self.kt_config, 'w+') as f:
                json.dump({}, f)

        try:
            with open(self.kt_config) as f:
                return json.load(f)
        except:
            log_red('kattis config maybe corrupted, resetting..')

        with open(self.kt_config, 'w+') as f:
            json.dump({}, f)
        return {}

    @abc.abstractmethod
    def _act(self) -> None:
        raise NotImplementedError()

    def act(self) -> None:
        """Main entry to execute the command
        """
        if self.REQUIRED_CONFIG:
            self.read_config_from_file()
        self._act()



import subprocess
from ..version import version
from ..base import Action
from ..logger import color_cyan, color_green, log, log_red
import requests
import sys

__all__ = ['Update']


class Update(Action):
    _PYPI_PACKAGE_INFO = 'https://pypi.org/pypi/kttool/json'

    def _act(self) -> None:
        pypi_info = requests.get(self._PYPI_PACKAGE_INFO)
        releases = list(pypi_info.json()['releases'])
        if len(releases) == 0:
            log_red('Hmm seems like there is currently no pypi releases :-?')
            return
        current_latest_version = releases[-1]
        if current_latest_version != version:
            subprocess.check_call(
                [
                    sys.executable, "-m", "pip", "install", "--upgrade",
                    "--no-cache-dir", f"kttool=={current_latest_version}"
                ]
            )
            log(
                f'Installed version {color_green(current_latest_version)} successfully!'
            )
        else:
            log(f'You already have the {color_green("latest")} version!')



from typing import List
from .actions.gen import Gen
from .actions.test import Test
from .actions.submit import Submit
from .actions.config import Config
from .actions.open import Open
from .actions.version import Version
from .actions.update import Update
from .base import Action

map_key_to_class = {
    'gen': Gen,
    'test': Test,
    'submit': Submit,
    'config': Config,
    'open': Open,
    'version': Version,
    'update': Update
}


def arg_parse(args: List[str]) -> Action:
    ''' Generate an appropriate command class based on user command stirng '''
    if len(args) == 0:
        raise ValueError(f'No command provided to kt')
    if args[0] not in map_key_to_class:
        raise ValueError(
            f'First argument should be one of {list(map_key_to_class.keys())}'
        )
    return map_key_to_class[args[0]](*args[1:])



import webbrowser

from ..base import Action
from ..logger import log

__all__ = ['Open']


class Open(Action):
    REQUIRED_CONFIG = True

    def _act(self) -> None:
        problem_url = self.get_problem_url()
        log(f'Openning {problem_url}')
        webbrowser.open(problem_url)



import re
import time
from dataclasses import dataclass
from typing import Any, Optional

import emoji
import requests
from bs4 import BeautifulSoup
from bs4.element import ResultSet
from reprint import output

from ..base import Action, require_login
from ..logger import color_cyan, color_green, color_red, log_green, log_red

__all__ = ['Submit']

AC_ICON = ':heavy_check_mark:'
RJ_ICON = ':heavy_multiplication_x:'
SK_ICON = ':white_medium_square:'


@dataclass
class SubmissionResult:
    submission_id: str
    submission_url: str


@dataclass
class SubmissionParseResult:
    test_cases: ResultSet
    overall_status: str
    run_time: str


@dataclass
class SubmissionVerdict:
    verdict: str
    num_test_cases: int
    ac_test_cases: int
    is_done: bool
    run_time: str
    is_rejected: bool


class Submit(Action):
    '''Handle kt submit action to push the file to kattis website'''
    REQUIRED_CONFIG = True

    @staticmethod
    def _parse_verdict(
        parsed_result: SubmissionParseResult
    ) -> SubmissionVerdict:
        num_test_cases = len(parsed_result.test_cases)
        ac_ct = 0
        is_ac = True
        rejected = finished = False
        _status = status = parsed_result.overall_status

        for res in parsed_result.test_cases:
            _class = res.get('class', None)
            if not _class:
                continue
            if 'is-accepted' in _class:
                ac_ct += 1
            elif 'is-empty' in _class:
                continue
            else:  # rejected
                rejected = True
                is_ac = False
                break

        finished = rejected or ac_ct == num_test_cases

        if status in {'Compiling', 'Running', 'New'}:
            ...
        elif status == 'Compile Error':
            _status = color_red(status)
        elif not finished:
            _status = color_cyan(status)
        elif is_ac:
            _status = color_green(status)
        else:
            _status = color_red(status)

        return SubmissionVerdict(
            verdict=_status,
            num_test_cases=num_test_cases,
            ac_test_cases=ac_ct,
            is_done=finished,
            run_time=parsed_result.run_time,
            is_rejected=rejected
        )

    def _is_finished(
        self, output_lines: output, submission_result: SubmissionResult,
        parsed_result: Optional[SubmissionParseResult]
    ) -> bool:
        """ Judge whether the result and status obtained from kattis submission
        page has indicated whether the solution judgement has been done

        Parameters
        ----------
        output_lines : output
            console object to print multiple lines inline
        result : SubmissionParseResult
            Parsed result objects

        Returns
        -------
        bool
            Whether if the task has been finished marking from kattis
        """
        if parsed_result is None:
            return False

        verdict: SubmissionVerdict = self._parse_verdict(parsed_result)

        res = [AC_ICON] * verdict.ac_test_cases
        if verdict.is_rejected:
            res.append(RJ_ICON)
        while len(res) < verdict.num_test_cases:
            res.append(SK_ICON)

        display_output = {
            'current time': time.strftime('%02l:%M%p %Z on %b %d, %Y'),
            'language': self.lang,
            'problem id': self._get_problem_id(),
            'running time': parsed_result.run_time,
            'submission id': submission_result.submission_id,
            'submission result': verdict.verdict,
            'test cases': emoji.emojize(' '.join(res), use_aliases=True)
        }
        for k, v in display_output.items():
            output_lines[k.ljust(20)] = str(v)

        return verdict.is_done

    @require_login
    def _query_submission_id_url(
        self, submission_result: SubmissionResult
    ) -> BeautifulSoup:
        page = self._request_get(submission_result.submission_url)
        soup = BeautifulSoup(page.content, 'html.parser')
        return soup

    def _parse_results_from_soup(
        self, soup: BeautifulSoup
    ) -> Optional[SubmissionParseResult]:

        try:
            submission_data = soup.find(
                'div', class_='status testcase testcase-row'
            )
            status_ret = soup.find(
                'div', class_='status is-status-accepted'
            ).next_element.next_element

            runtime_ret = status_ret.next_element
            return SubmissionParseResult(
                submission_data, status_ret.text, runtime_ret.text
            )
        except:
            ...

    def _render_result(self, submission_result: SubmissionResult) -> None:
        """ Continuously polling for result from `submission_url_ret`

        Parameters
        ----------
        submission_url_ret : str
            url for the submission to be checked
        """
        time_out = 60
        cur_time = 0
        done = False
        sleep_time = 0.4

        with output(output_type='dict') as output_lines:
            while cur_time < time_out:
                try:
                    soup: BeautifulSoup = self._query_submission_id_url(
                        submission_result
                    )
                    result = self._parse_results_from_soup(soup)
                    done = self._is_finished(
                        output_lines, submission_result, result
                    )
                    if done:
                        break
                except Exception as e:
                    log_red(f'Internal error: {e!r}')

                time.sleep(sleep_time)
                cur_time += sleep_time

    @require_login
    def _submit_code_file(self) -> Optional[SubmissionResult]:
        problem_id = self._get_problem_id()
        if not self._detect_code_files():
            return None
        data = {
            'submit': 'true',
            'submit_ctr': 2,
            'language': self.lang,
            'mainclass': '',
            'problem': problem_id,
            'tag': '',
            'script': 'true'
        }
        files = []
        with open(self.file_name) as sub_file:
            files.append(
                (
                    'sub_file[]', (
                        self.file_name.name, sub_file.read(),
                        'application/octet-stream'
                    )
                )
            )
        submit_url = self.get_url('submissionurl', 'submit')
        ret = self._request_post(submit_url, data=data, files=files)
        self._check_status_code(ret)
        submissions_base_url = self.get_url('submissionsurl', 'submissions')
        submit_response = ret.content.decode('utf-8').replace('<br />', '\n')
        submission_id = re.search(r'Submission ID: (\d+)',
                                  submit_response).group(1)
        return SubmissionResult(
            submission_id, f'{submissions_base_url}/{submission_id}'
        )

    def _check_status_code(self, ret: requests.Response):
        if ret.status_code != 200:
            if ret.status_code == 403:
                err = 'Access denied (403)'
            elif ret.status_code == 404:
                err = 'Incorrect submit URL (404)'
            else:
                err = f'Status code: {ret.status_code}'
            raise RuntimeError(f'Submission failed: {err}')

    def _act(self) -> None:
        '''Submit the code file for kattis judge'''
        submission_result: SubmissionResult = self._submit_code_file()
        if submission_result is None:
            return
        log_green(
            f'Submission successful -- id {submission_result.submission_id}'
        )
        self._render_result(submission_result)



from .context import supports_color

__all__ = [
    'color_cyan', 'color_green', 'color_red', 'log', 'log_green', 'log_cyan',
    'log_red', 'strike_through'
]

BOLD_SEQ = '\033[1m'
RESET_SEQ = '\033[0m'
BLACK = '\033[6;90m'
RED = '\033[6;91m'
GREEN = '\033[6;92m'
YELLOW = '\033[6;93m'
BLUE = '\033[6;94m'
MAGENTA = '\033[6;95m'
CYAN = '\033[6;96m'
WHITE = '\033[6;97m'


def color_cyan(text: str) -> str:
    if not supports_color:
        return text
    return f'{CYAN}{text}{RESET_SEQ}'


def color_green(text: str) -> str:
    if not supports_color:
        return text
    return f'{GREEN}{text}{RESET_SEQ}'


def color_red(text: str) -> str:
    if not supports_color:
        return text
    return f'{RED}{text}{RESET_SEQ}'


def strike_through(text: str) -> str:
    if not supports_color:
        return text
    result = ''
    for c in text:
        result = result + c + '\u0336'
    return result


log = print


def log_green(*args, **kwargs) -> None:
    log(color_green(*args, **kwargs))


def log_cyan(*args, **kwargs) -> None:
    log(color_cyan(*args, **kwargs))


def log_red(*args, **kwargs) -> None:
    log(color_red(*args, **kwargs))



import json
import os
import re
import shlex
import subprocess
import tempfile
import time
from collections import namedtuple
from dataclasses import dataclass
from pathlib import Path
from typing import List, Tuple

import psutil

from ..base import Action
from ..logger import (
    color_cyan, color_green, color_red, log, log_cyan, log_green, log_red,
    strike_through
)
from ..utils import (launch_subprocess, make_list_equal)

__all__ = ['Test']

AC = "Accepted".ljust(13, " ")
WA = "Wrong Answer".ljust(13, " ")


@dataclass
class Sample:
    index: int
    input_file: Path
    output_file: Path


class Test(Action):
    """Handle test command, run the code with local input and test against local output
    """
    REQUIRED_CONFIG = True

    def _compare_samples(self, samples: List[Sample]) -> None:
        rusage_denom = 1 << 20

        for sample in samples:
            is_ac = True
            actual = []
            expected = []
            diff = []
            try:
                with open(sample.output_file, 'r') as f:
                    expected = [l.strip(" \n") for l in f.readlines()]
                with open(sample.input_file, 'rb') as f:
                    raw_input_fixture = f.read()
                p = launch_subprocess(
                    shlex.split(f'{self.script} -'),
                    stdin=subprocess.PIPE,
                    stdout=subprocess.PIPE,
                    shell=False,
                    preexec_fn=os.setsid
                )
                proc = psutil.Process(p.pid)
                mem_used = proc.memory_info().rss / rusage_denom
                start_time = time.perf_counter()
                raw_output = p.communicate(raw_input_fixture)[0].decode()
                p.wait()
                taken = time.perf_counter() - start_time
                actual = [z.strip(" \n") for z in raw_output.split('\n')]
                make_list_equal(actual, expected)

                for i in range(len(expected)):
                    ''' 
                    Compare the values line by line
                    For each line, compare the values from left to right. 
                    '''
                    ith_line_exp = [z for z in expected[i].split(' ')]
                    ith_line_actual = [z for z in actual[i].split(' ')]

                    make_list_equal(ith_line_exp, ith_line_actual)
                    current_diff = ''
                    for j in range(len(ith_line_exp)):
                        lhs = ith_line_exp[j]
                        rhs = ith_line_actual[j]
                        is_good, now_diff = self._compare_entity(rhs, lhs)
                        is_ac &= is_good
                        current_diff += now_diff

                    diff.append(current_diff)

                if is_ac:
                    log_green(
                        f'Test Case #{sample.index}: {AC} ... {taken:.3f} s   {mem_used:.2f} M'
                    )
                else:
                    log_red(
                        f'Test Case #{sample.index}: {WA} ... {taken:.3f} s   {mem_used:.2f} M'
                    )
                    log_cyan('--- Input ---')
                    log(raw_input_fixture.decode())
                    log_cyan('--- Diff ---')
                    for i in range(len(diff)):
                        log(diff[i])

            except subprocess.CalledProcessError as e:
                log_red(f'Test case #{sample.index}: Runtime Error {e!r}')
            except Exception as e:
                self._record_unexpected_exception(sample, e)

    @staticmethod
    def _record_unexpected_exception(sample: Sample, ex: Exception) -> None:
        import traceback
        p = Path(tempfile.gettempdir())
        p.mkdir(parents=True, exist_ok=True)
        tmp_file = p / 'kt_test.log'
        with open(tmp_file, 'w+') as f:
            f.write(traceback.format_exc())
        log_red(
            f'Test case #{sample.index}: Internal Error {ex!r}. More info at {tmp_file}'
        )

    def _gather_samples(self) -> List[Sample]:
        input_files = [
            x for x in self.cwd.iterdir()
            if x.is_file() and x.stem.startswith('in')
        ]
        output_files = [
            x for x in self.cwd.iterdir()
            if x.is_file() and x.stem.startswith('ans')
        ]
        usable_samples: List[Sample] = []

        in_pattern = re.compile("in(\d+).txt")
        ans_pattern = re.compile("ans(\d+).txt")
        for input_file in input_files:
            idx = int(in_pattern.search(input_file.name).group(1))
            for output_file in output_files:
                if idx == int(ans_pattern.search(output_file.name).group(1)):
                    usable_samples.append(
                        Sample(
                            index=idx,
                            input_file=Path(input_file).absolute(),
                            output_file=Path(output_file).absolute()
                        )
                    )
                    break
        # run test from ascending number of file index
        return sorted(usable_samples, key=lambda x: x.index)

    def _act(self) -> None:
        """ Run the executable file against sample input and output files present in the folder
        The sample files will only be recognized if the conditions hold:
        - Naming style should be in{idx}.txt and ans{txt}.txt
        - for in{idx}.txt, there must exist a ans{idx}.txt with the same `idx`
        """
        if not self._detect_code_files():
            return

        # Get sample files that match the condition
        usable_samples = self._gather_samples()
        # run test
        log(f'Problem ID : {color_cyan(self._get_problem_id())}')
        log(f'Lanuage    : {self.lang}')
        if self.pre_script:
            log_cyan(f'running {self.pre_script}')
            subprocess.check_call(shlex.split(self.pre_script))

        self._compare_samples(usable_samples)

        if self.post_script:
            log_cyan(f'running {self.post_script}')
            subprocess.check_call(shlex.split(self.post_script))

    @staticmethod
    def _compare_entity(lhs: str, rhs: str) -> Tuple[bool, str]:
        if lhs == rhs:
            return True, f'{lhs} '
        return False, f'{color_red(strike_through(lhs))}{color_green(rhs)} '


import signal
import subprocess
import sys
from collections import namedtuple
from pathlib import Path
from typing import NoReturn, Union
import shlex
from .logger import color_cyan, log_green

__test_subprocesses = []

CATCH_PHRASE = 'Great is the art of beginning, but greater is the art of ending.'


def exit_gracefully(signum: signal.Signals, frame) -> NoReturn:
    global test_subprocesses
    original_sigint = signal.getsignal(signal.SIGINT)
    # restore the original signal handler as otherwise evil things will happen
    # in raw_input when CTRL+C is pressed, and our signal handler is not re-entrant
    signal.signal(signal.SIGINT, original_sigint)
    for sp in __test_subprocesses:
        try:
            sp.kill()
        except:
            pass
    log_green(CATCH_PHRASE)
    sys.exit(1)


def launch_subprocess(*args, **kwargs) -> subprocess.Popen:
    global __test_subprocesses
    p = subprocess.Popen(*args, **kwargs)
    __test_subprocesses.append(p)
    return p


def ask_with_default(qu: str, default_val: str = '') -> str:
    """ Print out `qu` to console and ask for input value from user
    If no input was provided by user, `default_val` will be returned instead

    Parameters
    ----------
    qu : str
        question to asked
    default_val : str, optional
        Default value to be used, by default ''

    Returns
    -------
    str
        string value as the response
    """
    qu = f'Please enter {color_cyan(qu)}'
    if default_val:
        qu = f'{qu} | Default value: {default_val}\n'
    ret = input(qu)
    if not ret:
        return default_val
    return ret


def make_list_equal(lhs: list, rhs: list, pad_element: str = '') -> None:
    """ Make two vector of string equation in length by padding with `pad_element`

    Parameters
    ----------
    lhs : list
        2 vectors of string to be made equal in length
    rhs : list
        2 vectors of string to be made equal in length
    pad_element : str, optional
        string to fill the shorter vector, by default ''
    """
    delta_size = abs(len(lhs) - len(rhs))
    delta_list = [pad_element] * delta_size
    if len(lhs) < len(rhs):
        lhs.extend(delta_list)
    else:
        rhs.extend(delta_list)


KATTIS_RC_URL = 'https://open.kattis.com/download/kattisrc'
HEADERS = {'User-Agent': 'kt'}

PLanguage = namedtuple(
    'ProgrammingLanguage',
    ['alias', 'extension', 'full_name', 'pre_script', 'script', 'post_script']
)

MAP_TEMPLATE_TO_PLANG = {
    'c':
        PLanguage(
            'c', 'c', 'C', 'gcc $%file%$.c -o $%file%$.out', './$%file%$.out',
            'rm $%file%$.out'
        ),
    'cpp':
        PLanguage(
            'cpp', 'cpp', 'C++',
            'g++ -std=c++14 -D_GLIBCXX_DEBUG -D_GLIBCXX_DEBUG_PEDANTIC -O3 $%file%$.cpp -o $%file%$.out',
            './$%file%$.out', 'rm $%file%$.out'
        ),
    'cc':
        PLanguage(
            'cc', 'cc', 'C++',
            'g++ -std=c++14 -D_GLIBCXX_DEBUG -D_GLIBCXX_DEBUG_PEDANTIC -O3 $%file%$.cc -o $%file%$.out',
            './$%file%$.out', 'rm $%file%$.out'
        ),
    'go':
        PLanguage(
            'go', 'go', 'Go', 'go build -o $%file%$', './$%file%$',
            'rm $%file%$'
        ),
    'java':
        PLanguage(
            'java', 'java', 'Java', 'javac *.java', './$%file%$', 'rm $%file%$'
        ),
    'js':
        PLanguage('js', 'js', 'JavaScript', '', 'node $%file%$.js', ''),
    'rs':
        PLanguage(
            'rs', 'rs', 'Rust', 'rustc $%file%$.rs', './$%file%$', 'rm $%file%$'
        ),
    'py2':
        PLanguage('py2', 'py', 'Python 2', '', 'python2 $%file%$.py', ''),
    'py3':
        PLanguage('py3', 'py', 'Python 3', '', 'python3 $%file%$.py', '')
}
